﻿TUI.Core.create({
    name:"TUI.map.MapabcImpl",
    extend:TUI.map.BaseMap,
    init:function () {
        this.createControl();                                // 创建地图控件
    },

    methods:{
// 创建地图
        createMap:function () {
            var self = this;
            var mapOptions = {
                level:4, //初始地图视野级别
                center:self._newLngLat(110.397, 36.917), //设置地图中心点
                doubleClickZoom:true, //双击放大地图
                scrollWheel:true//鼠标滚轮缩放地图
            }
            this._map = new MMap.Map(this._mapPanel, mapOptions);
            this._map.plugin(["MMap.ToolBar", "MMap.OverView", "MMap.Scale"], function () {
                toolbar = new MMap.ToolBar();
                toolbar.autoPosition = false; //加载工具条
                self._map.addControl(toolbar);
                scale = new MMap.Scale(); //加载比例尺
                self._map.addControl(scale);
            });
            this._map.setDefaultCursor('../images/openhand.cur');
        },
// 拉框放大
        dragMax:function () {
            this._map.setMouseTool('rectZoomIn');
        },
// 拉框缩小
        dragMin:function () {
            this._map.setMouseTool('rectZoomOut');
        },
// 移动地图
        moveMap:function () {
            this._map.setMouseTool('panWheelZoom'); // 鼠标移图

        },
// 测距
        range:function () {
            this._map.setMouseTool('rule'); //鼠标测距
        },
// 设置中心点
        setCenter:function (lng, lat, zoom) {
            var lngLat = this._newLngLat(lng, lat);
            this._map.setCenter(lngLat);
            this._map.setZoom(zoom);
        },
// 基本函数，显示全国图
        fullMap:function () {
            this.setCenter(110.397, 36.917, 5);
        },
// 锁定视窗
        lockWindowByLngLat:function (lngLat) {
            var lo = lngLat.lng, la = lngLat.lat;
            var bounds = this._map.getBounds();
            var left = bounds.southwest.lng;	//西南 经度
            var down = bounds.southwest.lat;	//西南 维度
            var right = bounds.northeast.lng;	//东北 经度
            var up = bounds.northeast.lat;	//东北 维度
            if (lo < left || lo > right || la < down || la > up) {
                this._map.setCenter(this._newLngLat(lo, la));
            }
        },
        // 最佳视窗效果
        _setFitview:function (overlays) {
            this._map.setFitView(overlays);
        },
/////////////////////////////////////内部私有函数///////////////////////////////////
        // 添加具体的覆盖物实例
        _addOverlay:function (overlay) {
            this._map.addOverlays(overlay);
        },
        // 移除具体的覆盖物实例
        _removeOverlay:function (overlay) {
            this._map.removeOverlays(overlay);
        },
        // 根据经度，纬度创建位置对象
        _newLngLat:function (lng, lat) {
            return new MMap.LngLat(lng, lat);
        },
        // 创建Marker
        _newMarkerContent:function(icon, label){
            var content = [];
            content.push('<div class="carMarker"><img src="{0}"/>'.format(icon));
            if (label)
                content.push('<span>{0}</span>'.format(label));
            content.push('</div>');
            return content.join("");
        },
        _newMarker:function (opt) {
            // 生成标签
            var marker = new MMap.Marker({
                position: opt.lngLat, // 位置
                content: this._newMarkerContent(opt.icon,opt.label),
                offset: new MMap.Pixel( 0 , 0 - opt.size.y / 2) //相对于基点的偏移量
            });
            return marker;
        },
        // 刷新Makert上的信息
        _refreshMarker:function( marker, opt ){
            if( marker ){
                if( opt.lngLat ){   // 如果指明了位置，就刷新位置
                    marker.setPosition(opt.lngLat);
                }
                if( opt.icon || opt.label ){
                    marker.setContent( this._newMarkerContent(opt.icon,opt.label) );
                }
            }
        },
        // 根据经度，纬度创建线
        _newPolyline:function (opt) {
            return new MMap.Polyline({ id: TUI.Event._id(), path:opt.lngLat,strokeColor:opt.color,strokeOpacity:0.7,strokeWeight:5});
        },

        // 绑定事件
        _bind:function(elem, event, fun){
            this._map.bind(elem, event, fun);
        },

        // 释放事件
        _unbind:function(elem, event, fun){
            this._map.unbind(elem, event, fun);
        },

        // 获取对象的轨迹
        _getPath: function(poly){
            return poly.getPath();
        },

        // 注册车辆信息窗口
        _addInfoWindow:function (marker, car, pos) {
            var self = this;
            this._safeBind(marker,"click", function(){
                if( self._infoWin ){   // 只开一个窗口
                    self._infoWin.close();
                }
                var html = self.getDetailCarTip(car.carNO, pos);
                var iw = new MMap.InfoWindow({
                    content:html,
                    autoMove:true,
                    size:new MMap.Size(350, 160),
                    offset:new MMap.Pixel(-115, -225)
                });
                iw.open(self._map, marker.getPosition());
                self.getAddress(pos.lo, pos.la, TUI.Event._id() );
                self._infoWin = iw;                          // 保存下窗体
                self._infoWin.carId = car.carId;              // 并且保存下窗体对应的空间对象
                self._map.bind(iw, 'close', function () {      // 关闭时清空
                    self._infoWin = null;
                });
            });
        },

        // 刷新车辆信息窗口
        _refreshInfoWindow:function (marker, car, pos) {
            var self = this;
            var iw = self._infoWin;
            if (iw) {
                var html = self.getDetailCarTip(car.carNO, pos);
                iw.setContent(html);
                iw.setPosition(marker.getPosition());
                self.getAddress(pos.lo, pos.la, TUI.Event._id());
            }
        },

        /////////////////////////////////////空间对象相关///////////////////////////////////
        // 绘制空间对象
        drawGeo:function(type, points, editable){
            var self = this;      // 获取全局对象
            var path = [];         // 已经接收的点
            var drawPath = null;  // 显示需要的点

            this._cleanBind(self._map);     // 清空所有事件
            this._cleanBind(self._geo);
            this.cleanGeo();                // 清空对象

            // 但如果只是显示，而不是编辑，则可以显示出来
            if( points && points.length > 0 && ( type == 0 || !editable ) ){
                var lngLats = this._parsePoints(points);
                _createGeo(lngLats, editable);   // 创建空间对象
                if( type == 0 )
                    this.setCenter( lngLats[0].lng, lngLats[0].lat, 12 );
                else
                    this._setFitview([this._geo]);  // 居中显示
            }
            // 否则，注册事件重新绘制
            else{
                self._map.setDefaultCursor("../images/cross_r.cur");
                self._safeBind( self._map, 'click', _clickCB);
                self._safeBind( self._map, 'mousemove', _moveCB);
                self._safeBind( self._map, 'rightclick', _rclickCB);
            }

            // 左键单击事件
            function _clickCB(event) {
                path.push(event.lnglat);                        // 往路径中加点
                drawPath = path.concat(path[path.length -1]);  // 显示的点总要多一个
                // 左键时，如果对象为空
                if( self._geo == null ){
                    _createGeo(drawPath);    // 以第一次单击位置创建对象
                    self._safeBind( self._geo, 'click', _clickCB);
                    self._safeBind( self._geo, 'mousemove', _moveCB);
                    self._safeBind( self._geo, 'rightclick', _rclickCB);
                }
                else{
                    if( type == 0 ){
                        self._geo.setPosition(event.lnglat);
                    }
                    else{
                        self._geo.setPath(drawPath);
                    }
                }
            }
            // 鼠标移动事件
            function _moveCB(event) {
                if( self._geo && type != 0 && drawPath.length > 1 ){
                    drawPath.pop();
                    drawPath.push(event.lnglat);
                    self._geo.setPath(drawPath); // 修改最后一个显示点的位置
                }
            }
            // 右键事件
            function _rclickCB(event) {
                self._cleanBind(self._map);
                self._cleanBind(self._geo);
                self._map.setDefaultCursor("../images/openhand.cur");
                if( self._geo  && type != 0 && path.length > 1){
                    self._geo.setPath(path);
                }
            }
            // 创建空间对象
            function _createGeo(lngLats, canEdit){
                // 如果是点
                if( type == 0 ){
                    self._geo = self._newMarker({
                        lngLat: lngLats[0],                     // 坐标
                        icon:"../images/marker.png",         // 图标
                        size:{x:25,y:25},                       // 图标大小
                        label: lg.markTip,                    // 标题
                        offset:{x:27,y:6},                     // 标题偏移
                        draggable:canEdit
                    });
                }
                // 如果是线
                else if( type == 1 ){
                    self._geo = new MMap.Polyline( {
                        path:lngLats,
                        strokeColor:"#FF0000",
                        fillColor:"#FF0000",
                        strokeWeight:2,
                        strokeOpacity:0.7,
                        fillOpacity:0.2,
                        strokeStyle:"dashed"
                    });
                }
                // 如果是面
                else if( type == 2 ){
                    self._geo = new MMap.Polygon( {
                        path:lngLats,
                        strokeColor:"#FF0000",
                        fillColor:"#FF0000",
                        strokeWeight:2,
                        strokeOpacity:0.7,
                        fillOpacity:0.2,
                        strokeStyle:"dashed"
                    });
                }
                self._addOverlay(self._geo);   // 添加到地图
            }
        }
    }
});